using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Internal;

namespace NTKDemo.Library.Logging
{
    internal class DemoLogger : ILogger
    {
        private readonly Action<LogLevel, string, object, Dictionary<string, object>, Exception> _logAction;
        private readonly string _categoryName;

        public DemoLogger(string categoryName, Action<LogLevel, string, object, Dictionary<string, object>, Exception> logAction)
        {
            _categoryName = categoryName;
            _logAction = logAction;
        }

        public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception exception, Func<TState, Exception, string> formatter)
        {
            var values = state as FormattedLogValues;
            if (values != null)
            {
                var template = values.Last();
                var v = new Dictionary<string, object>();
                for (var i = 0; i < values.Count-1; i++)
                {
                    v.Add(values[i].Key, values[i].Value);
                }
                _logAction(logLevel, _categoryName, template.Value, v, exception);
            }
            else
            {
                _logAction(logLevel, _categoryName, formatter(state, exception), new Dictionary<string, object>(), exception);
            }
        }

        public bool IsEnabled(LogLevel logLevel)
        {
            return true;
        }

        public IDisposable BeginScope<TState>(TState state)
        {
            return null;
        }
    }
}