﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;

namespace NTKDemo.Library
{
    public class SmartHomeApi : ISmartHomeApi
    {
        private readonly ILogger<SmartHomeApi> _logger;

        public SmartHomeApi(ILoggerFactory loggerFactory)
        {
            _logger = loggerFactory.CreateLogger<SmartHomeApi>();
        }

        public async Task<IList<RoomTemperatureSensor>> ReadAndAdjustTemperatureValuesAsync()
        {
            var heatPumpTemperature = await ReadHeatPumpTemperatureAsync();

            _logger.LogInformation("Reading room temperature sensors...");

            var sensors = await ReadRoomTemperatureSensorsAsync();
            #region Demo2
            using (_logger.BeginScope("Heat pump temperature is {HeatPumpTemperature}", heatPumpTemperature))
            #endregion
            {
                foreach (var sensor in sensors)
                {
                    _logger.LogInformation("Room temperature in {Sensor} is {Temperature}.", sensor.Name, sensor.Temperature);

                    if (sensor.Temperature < 20)
                    {
                        _logger.LogInformation("Adjusting temperature for {Sensor} to higher.", sensor.Name);
                        await AdjustTemperatureAsync(sensor.Name, TemperatureAdjustment.Higher);
                    }
                    else if (sensor.Temperature > 22)
                    {
                        _logger.LogInformation("Adjusting temperature for {Sensor} to lower.", sensor.Name);
                        await AdjustTemperatureAsync(sensor.Name, TemperatureAdjustment.Lower);
                    }
                }
            }
            return sensors;
        }




        #region Demo
        private Task AdjustTemperatureAsync(string sensorName, TemperatureAdjustment adjustment)
        {
            if (sensorName == "Office")
            {
                _logger.LogWarning("Temperature in {Sensor} could not be adjusted. Possible malfunction.", sensorName);
            }

            return Task.CompletedTask;
        }
        #endregion

        #region Demo
        private Task<double> ReadHeatPumpTemperatureAsync()
        {
            return Task.FromResult(42.4);
        }


        private Task<List<RoomTemperatureSensor>> ReadRoomTemperatureSensorsAsync()
        {
            return Task.FromResult(
            new List<RoomTemperatureSensor>
            {
                new RoomTemperatureSensor {Name = "Bedroom", Temperature = 23.5},
                new RoomTemperatureSensor {Name = "Living room", Temperature = 24.2},
                new RoomTemperatureSensor {Name = "Kitchen", Temperature = 23.9},
                new RoomTemperatureSensor {Name = "Children 1", Temperature = 21.3},
                new RoomTemperatureSensor {Name = "Children 2", Temperature = 22.1},
                new RoomTemperatureSensor {Name = "Office", Temperature = 19.8},
            });
        }
#endregion
    }

    public enum TemperatureAdjustment
    {
        Higher, Lower
    }

    public class RoomTemperatureSensor
    {
        public string Name { get; set; }
        public double Temperature { get; set; }
    }

    public static class LogMessages
    {
        private static readonly Action<ILogger, string, double, Exception> 
            RoomTemperatureStatusDefinition = LoggerMessage.Define<string, double>(
                LogLevel.Information,
                new EventId(1, nameof(RoomTemperatureStatus)),
                "Room temperature in {Sensor} is {Temperature}.");

        public static void RoomTemperatureStatus(this ILogger logger, string sensorName, double temperature)
        {
            RoomTemperatureStatusDefinition(logger, sensorName, temperature, null);
        }
    }
}